package ap;

import ap.proof.{ConstraintSimplifier, ProofTree, ModelSearchProver}
import ap.terfor.conjunctions.{Conjunction, Quantifier}
import ap.parameters.GoalSettings

object IntelliFileProver {
  
  abstract class Result
  case class Proof(tree : ProofTree, model : Conjunction) extends Result
  case object NoProof extends Result
  case class CounterModel(counterModel : Conjunction) extends Result
  case object NoCounterModel extends Result
  case object SearchCancelled extends Result
  
}

/**
 * A prover that decides, depending on the kind of the problem, whether it
 * should try to construct a proof tree or just search for counterexamples
 */
class IntelliFileProver(reader : java.io.Reader,
                        simplifier : ConstraintSimplifier,
                        // a timeout in milliseconds
                        timeout : Int,
                        output : Boolean,
                        userDefStoppingCond : => Boolean,
                        goalSettings : GoalSettings)
      extends AbstractFileProver(reader, output, timeout,
                                 userDefStoppingCond, goalSettings) {

  lazy val proofTree : ProofTree = constructProofTree(simplifier)

  lazy val counterModel : ModelSearchProver.Result = findCounterModel

  val result : IntelliFileProver.Result = {
    if ((formula.constants subsetOf eliminatedConstants) &&
        (Conjunction.collectQuantifiers(formula) subsetOf Set(Quantifier.ALL))) {
      // try to find a countermodel
      counterModel match {
        case ModelSearchProver.NoModel =>
          IntelliFileProver.NoCounterModel
        case ModelSearchProver.FoundModel(m) =>
          IntelliFileProver.CounterModel(m) 
        case ModelSearchProver.SearchCancelled =>
          IntelliFileProver.SearchCancelled
      }
    } else {
      // try to construct a proof
      val model = findModel(proofTree.closingConstraint)
      if (model.isFalse)
        IntelliFileProver.NoProof
      else
        IntelliFileProver.Proof(proofTree, model)
    }
  }
  
}
