package ap;

import ap.proof.{TestProofTree, ConstraintSimplifier}
import ap.util.{Debug, Logic, APTestCase, PlainRange, FilterIt}
import ap.parameters.{GoalSettings, Param}

class TestProver(n : String) extends APTestCase(n) {

  def runTest = {
    n match {
      case "testFilesTrue" => testFilesTrue
      case "testFilesFalse" => testFilesFalse
      case "testFilesCountermodel" => testFilesCountermodel
    }
  }

  private val testcaseDir = new java.io.File ("testcases/")
  private val ctrueDir = new java.io.File (testcaseDir, "constraint_true")
  private val cfalseDir = new java.io.File (testcaseDir, "constraint_false")
  private val countermodelExistsDir = new java.io.File (testcaseDir, "countermodel_exists")
  
  private val goalSettings = Param.POS_UNIT_RESOLUTION.set(GoalSettings.DEFAULT, true)
  
  /**
   * Try to prove some problems which should be provable, using a timeout of 20000
   */
  def testFilesTrue = {
    for (file <- FilterIt(ctrueDir.listFiles.elements,
                          (f:java.io.File) => f.isFile)) {
      val prover = new FileProver(new java.io.FileReader(file),
                                  ConstraintSimplifier.ALL_SIMPLIFIER, 20000,
                                  false, goalSettings)
      assert(prover.proofTree.closingConstraint.isTrue, file)
//      TestProofTree.assertNormalisedTree(prover.proofTree)
    }
  }
 
  /**
   * Try to prove some problems which should not be provable,
   * using a timeout of 15000
   */
  def testFilesFalse = {
    for (file <- FilterIt(cfalseDir.listFiles.elements,
                          (f:java.io.File) => f.isFile)) {
      val prover = new FileProver(new java.io.FileReader(file),
                                  ConstraintSimplifier.ALL_SIMPLIFIER, 15000,
                                  false, goalSettings)
      assert(prover.proofTree.closingConstraint.isFalse, file)
//      TestProofTree.assertNormalisedTree(prover.proofTree)
    }
  }

  /**
   * Try to construct the countermodel of some invalid problems
   */
  def testFilesCountermodel = {
    for (file <- FilterIt(countermodelExistsDir.listFiles.elements,
                          (f:java.io.File) => f.isFile)) {
      val prover = new FileModelSearcher(new java.io.FileReader(file), false,
                                         goalSettings)
      assert(!prover.counterModel.isFalse, file)
    }
  }
}
