package ap.proof;

import ap.proof.goal.{PrioritisedTask, Goal, TaskManager}
import ap.terfor.{Formula, ConstantTerm, TermOrder}
import ap.terfor.conjunctions.{Conjunction, NegatedConjunctions, Quantifier}
import ap.terfor.substitutions.Substitution

abstract class ProofTreeFactory {

  def and(subtrees : Seq[ProofTree], order : TermOrder) : ProofTree
  
  def quantify(subtree : ProofTree,
               quan : Quantifier,
               quantifiedConstants : Set[ConstantTerm],
               order : TermOrder,
               subtreeOrder : TermOrder) : ProofTree
  
  def substitute(subtree : ProofTree,
                 subst : Substitution,
                 order : TermOrder,
                 subtreeOrder : TermOrder) : ProofTree
               
  def weaken(subtree : ProofTree,
             disjunct : Conjunction,
             order : TermOrder) : ProofTree

  /**
   * A callback to tell that a constant has been eliminated. Upon elimination,
   * it must be possible to provide a witness, i.e., given a substitution that
   * describes concrete values for all the remaining variables, a solution
   * for <code>c</code> must be computable.
   */
  def eliminatedConstant(subtree : ProofTree,
                         c : ConstantTerm,
                         witness : (Substitution, TermOrder) => Substitution,
                         order : TermOrder) : ProofTree
             
  //////////////////////////////////////////////////////////////////////////////
  
  def updateGoal(updatedFacts : Conjunction,
                 updatedClauses : NegatedConjunctions,
                 updatedElimConstants : Set[ConstantTerm],
                 updatedOrder : TermOrder,
                 newTasks : Iterable[PrioritisedTask],
                 goal : Goal) : ProofTree

  def updateGoal(updatedElimConstants : Set[ConstantTerm],
                 updatedOrder : TermOrder,
                 newTasks : Iterable[PrioritisedTask],
                 goal : Goal) : ProofTree =
    updateGoal(goal.facts, goal.clauses, updatedElimConstants, updatedOrder,
               newTasks, goal)

  def updateGoal(updatedFacts : Conjunction,
                 newTasks : Iterable[PrioritisedTask],
                 goal : Goal) : ProofTree =
    updateGoal(updatedFacts, goal.clauses, goal.eliminatedConstants,
               goal.order, newTasks, goal)

  def updateGoal(updatedFacts : Conjunction, goal : Goal) : ProofTree =
    updateGoal(updatedFacts, goal.clauses, goal.eliminatedConstants,
               goal.order, List(), goal)

  def updateGoal(updatedClauses : NegatedConjunctions,
                 goal : Goal) : ProofTree =
    updateGoal(goal.facts, updatedClauses, goal.eliminatedConstants,
               goal.order, List(), goal)

  def updateGoal(updatedClauses : NegatedConjunctions,
                 newTasks : Iterable[PrioritisedTask],
                 goal : Goal) : ProofTree =
    updateGoal(goal.facts, updatedClauses, goal.eliminatedConstants,
               goal.order, newTasks, goal)

  def updateGoal(newTasks : Iterable[PrioritisedTask], goal : Goal) : ProofTree =
    updateGoal(goal.facts, goal.clauses, goal.eliminatedConstants, goal.order,
               newTasks, goal)

  def updateGoal(goal : Goal) : ProofTree =
    updateGoal(goal.facts, goal.clauses, goal.eliminatedConstants, goal.order,
               List(), goal)

  /**
   * Possibly exchange the whole task manager or update arbitrary tasks
   */
  def updateGoal(tasks : TaskManager, goal : Goal) : ProofTree

}
