package ap.terfor.preds;

import ap.terfor.substitutions.VariableShiftSubst
import ap.util.{Debug, Seqs, LazyMappedSet, UnionSet}

object ReduceWithPredLits {
  
  private val AC = Debug.AC_PROPAGATION
  
  def apply(positiveLits : scala.collection.Set[Atom],
            negativeLits : scala.collection.Set[Atom],
            order : TermOrder) : ReduceWithPredLits =
    new ReduceWithPredLits(positiveLits, negativeLits, order)
  
  def apply(conj : PredConj, order : TermOrder) : ReduceWithPredLits = {
    ////////////////////////////////////////////////////////////////////////////
    Debug.assertPre(AC, conj isSortedBy order)
    ////////////////////////////////////////////////////////////////////////////
    ReduceWithPredLits(conj.positiveLitsAsSet, conj.negativeLitsAsSet, order)
  }
    
}

class ReduceWithPredLits private (positiveLits : scala.collection.Set[Atom],
                                  negativeLits : scala.collection.Set[Atom],
                                  order : TermOrder) {

  def addLits(furtherLits : PredConj) : ReduceWithPredLits =
    if (furtherLits.isTrue)
      this
    else
      ReduceWithPredLits(UnionSet(positiveLits, furtherLits.positiveLitsAsSet),
                         UnionSet(negativeLits, furtherLits.negativeLitsAsSet),
                         order)

  /**
   * Create a <code>ReduceWithEqs</code> that can be used underneath
   * <code>num</code> binders. The conversion of de Brujin-variables is done on
   * the fly, which should give a good performance when the resulting
   * <code>ReduceWithEqs</code> is not applied too often (TODO: caching)
   */
  def passQuantifiers(num : Int) : ReduceWithPredLits = {
    val upShifter = VariableShiftSubst.upShifter[Atom](num, order)
    val downShifter = VariableShiftSubst.downShifter[Atom](num, order)
    ReduceWithPredLits(new LazyMappedSet(positiveLits, upShifter, downShifter),
                       new LazyMappedSet(negativeLits, upShifter, downShifter),
                       order)
  }

  def apply(conj : PredConj) : PredConj = {
    ////////////////////////////////////////////////////////////////////////////
    Debug.assertPre(ReduceWithPredLits.AC, conj isSortedBy order)
    ////////////////////////////////////////////////////////////////////////////
    // TODO: should be done using binary search
    if (!Seqs.disjoint(positiveLits, conj.negativeLits) ||
        !Seqs.disjoint(negativeLits, conj.positiveLits)) {
      // a contradiction has been found
      PredConj.FALSE(conj)
    } else {
      conj.updateLitsSubset(conj.positiveLits filter ((a) => !(positiveLits contains a)),
                            conj.negativeLits filter ((a) => !(negativeLits contains a)),
                            order)
    }
  }
  
}
