package ap.util;

object UnionSet {
  
  private val AC = Debug.AC_SET_UTILS

  def apply[A](set1 : scala.collection.Set[A],
               set2 : scala.collection.Set[A]) : scala.collection.Set[A] =
    if (set1.size == 0)
      set2
    else if (set2.size == 0)
      set1
    else
      new UnionSet (set1, set2)
}

/**
 * A (lazy) <code>Set</code> that represents the union of two disjoint
 * <code>Set</code>s
 */
class UnionSet[A] private (set1 : scala.collection.Set[A],
                           set2 : scala.collection.Set[A])
      extends scala.collection.Set[A] {

  def contains(x : A) : Boolean = (set1 contains x) || (set2 contains x)
  
  def size : Int = {
    ////////////////////////////////////////////////////////////////////////////
    Debug.assertInt(UnionSet.AC, Seqs.disjoint(set1, set2))
    ////////////////////////////////////////////////////////////////////////////
    set1.size + set2.size
  }
  
  def elements : Iterator[A] = set1.elements ++ set2.elements
  
}
